package cologne.eck.all_peas.gui;



import java.awt.Color;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.FontUIResource;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.tools.UnexpectedValueException;



@SuppressWarnings("serial")
public class ViewMenu extends JMenu implements ActionListener {
	
	private ResourceBundle languagesBundle;
	
	FileTypePanel ftp;

	public ViewMenu(FileTypePanel _ftp, boolean plainModus) {
		
		Font font = new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize());
		
		ftp = _ftp;
		languagesBundle = PeaProperties.getBundle();
		this.setFont(font);
		this.setText(languagesBundle.getString("view"));
		this.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("view_mnemonic")).getKeyCode());
//		int fontSize = PswDialogView.getFontsize();
//		this.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		//help.setMnemonic(KeyEvent.VK_H);
		//this.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("view_mnemonic")).getKeyCode());
		
		JMenuItem treeViewItem = new JMenuItem(languagesBundle.getString("tree_view"));
		treeViewItem.setActionCommand("treeView");
		treeViewItem.setFont(font);
		treeViewItem.setToolTipText(languagesBundle.getString("tooltip_tree_view"));
		//instructionItem.setMnemonic(KeyEvent.VK_I);
		//treeViewItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		treeViewItem.addActionListener(this);
		this.add(treeViewItem);
		
		JMenuItem fullNameItem = new JMenuItem(languagesBundle.getString("full_name"));
		fullNameItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component files with the full path.");
		fullNameItem.setActionCommand("fullName");
		fullNameItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		//fullNameItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("web_mnemonic")).getKeyCode());
		fullNameItem.addActionListener(this);
		this.add(fullNameItem);

		if (plainModus == false){
			JMenuItem checkAgainItem = new JMenuItem(languagesBundle.getString("refresh"));
			checkAgainItem.setActionCommand("refresh");
			checkAgainItem.setToolTipText(languagesBundle.getString("tooltip_refresh"));
			checkAgainItem.setFont(font);
			checkAgainItem.addActionListener(this);
			this.add(checkAgainItem);		
		}

		JMenuItem cleanupItem = new JMenuItem(languagesBundle.getString("cleanup"));
		cleanupItem.setActionCommand("cleanup");
		cleanupItem.setFont(font);
		cleanupItem.setToolTipText(languagesBundle.getString("tooltip_cleanup"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		cleanupItem.addActionListener(this);
		this.add(cleanupItem);
		
		JMenuItem increaseFontSizeItem = new JMenuItem(languagesBundle.getString("enlarge_font"));//languagesBundle.getString("cleanup"));
		increaseFontSizeItem.setActionCommand("increaseFontSize");
		increaseFontSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component increases the font size for this program.");
		increaseFontSizeItem.setFont(font);
//		increaseFontSizeItem.setToolTipText(languagesBundle.getString("enlarge_font"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		increaseFontSizeItem.addActionListener(this);
		this.add(increaseFontSizeItem);
		
		JMenuItem decreaseFontSizeItem = new JMenuItem(languagesBundle.getString("reduce_font"));//languagesBundle.getString("cleanup"));
		decreaseFontSizeItem.setActionCommand("decreaseFontSize");
		decreaseFontSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component decreases the font size for this program.");
		decreaseFontSizeItem.setFont(font);
//		increaseFontSizeItem.setToolTipText(languagesBundle.getString("tooltip_cleanup"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		decreaseFontSizeItem.addActionListener(this);
		this.add(decreaseFontSizeItem);
		
		JMenuItem colorItem = new JMenuItem(languagesBundle.getString("background_color"));//languagesBundle.getString("cleanup"));
		colorItem.setActionCommand("setColor");
		colorItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a color chooser to set the background color for this program.");
		colorItem.setFont(font);
//		increaseFontSizeItem.setToolTipText(languagesBundle.getString("tooltip_cleanup"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		colorItem.addActionListener(this);
		this.add(colorItem);
	}

	/**
	 * Call for every child of a container updateUI()
	 * 
	 * @param c	the container to update
	 */
/*	public static void updateAllComponents(Container c) {
	    Component[] comps = c.getComponents();
	    int len = comps.length;
	   // ArrayList<Component> compList = new ArrayList<Component>();
	    for (int i = 0;i < len; i++) {
	        //compList.add(comp);
	        if (comps[i] instanceof Container) {
        		((JComponent)comps[i]).updateUI();
        		System.out.println(comps[i].getClass());
	        	updateAllComponents((Container)comps[i]);
	            //compList.addAll(updateAllComponents((Container) comp));
	        } else {
	        	try {
	        		((JComponent)comps[i]).updateUI();
	        		System.out.println(comps[i].getClass());
	        	} catch (Exception e) {
	        		System.out.println(e);
	        	}
	        }
	    }
	}*/

	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
//		System.out.println(command);
		FileComposer fc = ftp.getFileComposer();
		
		if (command.equals("increaseFontSize")) {
			
			// change font size
			int oldFontSize = PeaProperties.getFontSize();
			int newFontSize = oldFontSize;
			if (oldFontSize < 18) {
				newFontSize = oldFontSize + 2;
			PeaProperties.setFontSize( newFontSize);
			} 
		    
			// set font size (UIManager)
			FontUIResource f = new FontUIResource(Font.SANS_SERIF, Font.PLAIN, newFontSize);			
			java.util.Enumeration<Object> keys = UIManager.getDefaults().keys();
		    while (keys.hasMoreElements())  {
		        Object key = keys.nextElement();
		        Object value = UIManager.get(key);
		        if (value instanceof javax.swing.plaf.FontUIResource) {
		            UIManager.put(key, f);
		        }
		    }
		   // UIManager.put("OptionPane.messageFont", f);
		   // UIManager.put("OptionPane.buttonFont", f); 
		    // resizes currently shown items, labels...
		    //this.getParent().revalidate();
		    //this.getParent().repaint();
		    UIManager.getLookAndFeelDefaults()
	        .put("defaultFont", new Font(Font.SANS_SERIF, Font.PLAIN, newFontSize));

			if (SwingUtilities.getWindowAncestor(this) instanceof JFrame){ // LockFrame: plain text mode
				PswDialogView.setUI(false);
			} else if (SwingUtilities.getWindowAncestor(this) instanceof JDialog){ // Password dialog: encrypted mode
				PswDialogView.setUI(true);
			} else {
				new UnexpectedValueException("window ancgestor", "Window",
						"is not JDialog or JFrame: " + SwingUtilities.getWindowAncestor(this).toString()).printDescription();
			}
			Window win = SwingUtilities.windowForComponent(this);
			SwingUtilities.updateComponentTreeUI(win);
			win.pack();
			win.setVisible(true);
			


		/*    SwingUtilities.updateComponentTreeUI(PswDialogView.getView());
		    PswDialogView.getView().pack();
		    PswDialogView.getView().setVisible(true); */
		    		    
		} else if (command.equals("decreaseFontSize")) {
			
			// change font size
			int oldFontSize = PeaProperties.getFontSize();
			int newFontSize = oldFontSize;
			if (oldFontSize > 10) {
				newFontSize = oldFontSize - 2;
			PeaProperties.setFontSize( newFontSize);
			} 
			
			// set font size (UIManager)
			FontUIResource f = new FontUIResource(Font.SANS_SERIF, Font.PLAIN, newFontSize);			
			java.util.Enumeration<Object> keys = UIManager.getDefaults().keys();
		    while (keys.hasMoreElements())  {
		        Object key = keys.nextElement();
		        Object value = UIManager.get(key);
		        if (value instanceof javax.swing.plaf.FontUIResource) {
		            UIManager.put(key, f);
		        }
		    }  
		  //  UIManager.put("OptionPane.messageFont", f);
		  //  UIManager.put("OptionPane.buttonFont", f); 
		    // resizes currently shown items, labels...
		    UIManager.getLookAndFeelDefaults()
	        .put("defaultFont", new Font(Font.SANS_SERIF, Font.PLAIN, newFontSize));
		    
		    // update (current) view
		    SwingUtilities.updateComponentTreeUI(PswDialogView.getView());		    
//		    updateAllComponents(PswDialogView.getView().getContentPane());		    
			if (SwingUtilities.getWindowAncestor(this) instanceof JFrame){ // LockFrame: plain text mode
				PswDialogView.setUI(false);
			} else if (SwingUtilities.getWindowAncestor(this) instanceof JDialog){ // Password dialog: encrypted mode
				PswDialogView.setUI(true);
			} else {
				new UnexpectedValueException("window ancgestor", "Window",
						"is not JDialog or JFrame: " + SwingUtilities.getWindowAncestor(this).toString()).printDescription();
			}
			Window win = SwingUtilities.windowForComponent(this);
			SwingUtilities.updateComponentTreeUI(win);
			win.pack();
			win.setVisible(true);
	/*	    SwingUtilities.updateComponentTreeUI(PswDialogView.getView());
		    PswDialogView.getView().pack();
		    PswDialogView.getView().setVisible(true);*/
		    
		} else if (command.equals("treeView")){
			// add zip buttons as empty strings:
			String[] newTexts = fc.showFileHierarchy(ftp.getValidFileNames(false, false));
			if (newTexts == null) {
				return;
			}
			// set the new texts as check box texts
			ftp.updateCheckBoxTexts(newTexts);
			ftp.updateWindow();

		} else if (command.equals("fullName")){
			ftp.showFullFileNames();
			ftp.updateWindow();

		} else if (command.equals("refresh")){
			
	/*		if (fc.getPlainModus() == true){
				// do not check wrong password or wrong salt files
				x
			} */
			
			fc.updateFiles(true);
			ftp.getFileComposer().getFileModel().calculateNumberAndSize(ftp.getValidFileNames(true, false));
			ftp.updateNumberAndSize();
			ftp.updateWindow();

		} else if (command.equals("cleanup")) {
			if (ftp.getFileComposer().isPlainModus() == true) {
				String[] unselectedFiles = ftp.getUnselectedValidFileNames();
				if (unselectedFiles != null) {
					StringBuilder unselectedFilesBuilder = new StringBuilder();
					for (int i = 0; i < unselectedFiles.length; i++) {
						unselectedFilesBuilder.append(unselectedFiles[i]);
						unselectedFilesBuilder.append("\n");
					}
					String unselectedFilesString = new String(unselectedFilesBuilder);
					// display a warning and an option to break
					int result = JOptionPane.showConfirmDialog(ftp,
							PeaProperties.getBundle().getString("leave_files_unencrypted")
							+ "\n" + unselectedFilesString,
							PeaProperties.getBundle().getString("warning"),
							JOptionPane.WARNING_MESSAGE,
							JOptionPane.YES_NO_OPTION);
					if (result != 0) {	//yes = 0, 
						return;
					} 			
				}
			}
			ftp.cleanupView();
			ftp.getFileComposer().getFileModel().cleanupMap();
			ftp.getFileComposer().getFileModel().calculateNumberAndSize(ftp.getValidFileNames(true, false));
			ftp.updateNumberAndSize();
			ftp.updateWindow();
			
		} else if (command.equals("setColor")) {
			Color color = JColorChooser.showDialog(this,languagesBundle.getString("background_color"), PeaProperties.getPeaColor());
			if (color == null) {
				return;
			}
			// check darkness:
			int darkness = ((30 * color.getRed() +
		            59 * color.getGreen() +
		            11 * color.getBlue()) / 100);
			while (darkness < 210) { // make brighter
				color = color.brighter();
				int newDarkness = ((30 * color.getRed() +
		            59 * color.getGreen() +
		            11 * color.getBlue()) / 100);
				if (newDarkness == darkness) { // this occurs if colors are 0
					int r = color.getRed();
					int g = color.getGreen();
					int b = color.getBlue();
					if ( r < 50) {
						r = r + 50;
					}
					if (g < 50) {
						g = g + 50;
					}
					if (b < 50) {
						b = b + 50;
					}
					color = new Color(r,g,b);
					darkness = ((30 * color.getRed() +
				            59 * color.getGreen() +
				            11 * color.getBlue()) / 100);
				} else {
					darkness = newDarkness;
				}
			}
			PeaProperties.setPeaColor(color);
			if (SwingUtilities.getWindowAncestor(this) instanceof JFrame){ // LockFrame: plain text mode
				PswDialogView.setUI(false);
			} else if (SwingUtilities.getWindowAncestor(this) instanceof JDialog){ // Password dialog: encrypted mode
				PswDialogView.setUI(true);
			} else {
				new UnexpectedValueException("window ancgestor", "Window",
						"is not JDialog or JFrame: " + SwingUtilities.getWindowAncestor(this).toString()).printDescription();
			}
			
			Window win = SwingUtilities.windowForComponent(this);
			SwingUtilities.updateComponentTreeUI(win);
			win.pack();
			win.setVisible(true);
			PswDialogView.getView().validate();
			PswDialogView.getView().repaint();
			
		    //PswDialogView.getView().validate();	    
	/*	    SwingUtilities.updateComponentTreeUI(PswDialogView.getView());
		    PswDialogView.getView().pack();
		    PswDialogView.getView().setVisible(true);
		    PswDialogView.getView().validate();
		    PswDialogView.getView().invalidate();
		    PswDialogView.getView().repaint();*/
			//PswDialogView.getView().paint(PswDialogView.getView().getGraphics());
			
		} else {
			System.out.println("Invalid command in help menu");
		}
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleViewMenu();
        }
        return accessibleContext;
    }

    protected class AccessibleViewMenu extends AccessibleJMenu {
        //Inherit everything, override nothing.
    }
}
