package cologne.eck.file_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Dialog to get the password and start the file pea.
 */



import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;

import javax.swing.SwingUtilities;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.tools.PropertyHandler;
//import cologne.eck.tools.Help;
import cologne.eck.tools.UnexpectedValueException;



public final class FileControl extends PeaControl {

	private static FileControl pswDialog; // static instance
	private static LockFrameFile lockFrame; // associated static instance of LockFrameFile
	private static PswDialogView dialogView; // associated instance of PswDialogView
	private static String peaPropFileName = "file.properties";
		
	/**
	 * used for ShutDownHook (if program is not closing correctly)
	 * true = unencrypted files remaining (something to do for ShutDownHook) 
	 * false = all files encrypted (nothing to do for ShutDownHook)
	 */
	private static volatile boolean decrypted = false; 
	
	private static final String VERSION = "1.1";
	private static final String YEAR_OF_PUBLICATION = "2019"; 
	private static final String WEBSITE = "/html/file_pea.html";
	private static final String SOURCE_LINK = "https://opensourceprojects.eu/p/filelockpea/code/2/tree/"; 

	private FileTypePanel decryptedFileTypePanel;	
	
	
	private FileControl() {	
		pswDialog = this;
		setDialog(pswDialog);
		dialogView = PswDialogView.getInstance();
	}

	public final static FileControl getInstance() {
		if (pswDialog == null) {
			pswDialog = new FileControl();
		} else {
			//return null
		}
		return pswDialog;
	}	

	/**
	 * start the PEA
	 * 
	 * @param args	test mode: "-t" or rescue mode: "-r"
	 */
	public static void main(String[] args) {		
				

		
		PeaControl.setPeaPropFileName(peaPropFileName);
		new PropertyHandler().setAllPropValues();

		// set working mode: -t for test mode: write stderr to log file
		if(args.length > 0) {
			PeaProperties.setWorkingMode(args[0]);
		}

		// encrypt open file if shutdown without closing the program:
		ShutdownHookThread hook = new ShutdownHookThread();
		Runtime.getRuntime().addShutdownHook(hook);

		PeaControl.initializeVariables();		
		PeaProperties.setFileType("file");			

		SwingUtilities.invokeLater(new EventDispatchThread());
	}

	
	// indicate that the opened files are actually encrypted or plain text
	public final static void setDecrypted(boolean _decrypted) {
		decrypted = _decrypted;
	}

	
	//-------------------HELPER-FUNCTIONS--------------		

	@Override
	public void preComputeInThread() {
		//getCipherBytes(encryptedFileName);		
	}

	@Override
	public void clearSecretValues() {
		// keys are necessary to encrypt files later, so do not clear
	}

	@Override
	public void startDecryption() {

		byte[] keyMaterial = null;
		boolean printErrors = false;
		if (PeaProperties.getWorkingMode().equals("-t")) {
			printErrors = true;
		}		

		// create the panel, composer and model for decryptedFiles
		// TODO test
		if (decryptedFileTypePanel == null) {
			FileModel decryptedFileModel = new FileModel();
			FileComposer decryptedFileComposer = new FileComposer(decryptedFileModel);
			decryptedFileTypePanel = new FileTypePanel(300,300, true, false, decryptedFileComposer, lockFrame);
		}		
		
		// set FileTypePanel for LockFrame:
		LockFrameFile.setFileDisplayPanel(decryptedFileTypePanel);

		// create the frame: 		
		lockFrame = LockFrameFile.getInstance(
				PswDialogView.getView().getLocation(), 
				PeaProperties.getFrameDim());//PswDialogView.getView().getSize());		

		// the selected file names to decrypt
		String[] fileNames = null;

		try {

			if ( PswDialogView.isInitializing() == true ){
				System.out.println("Initialization");
				decryptedFileTypePanel.getFileComposer().setPlainModus(true); // files are encrypted

				// option: try to get salt from current encrypted files
				setSaltForNewFiles();

				keyMaterial = getKeyMaterial();

				if (keyMaterial == null) { // bug
					new UnexpectedValueException("keyMaterial", "byte[]", "is null").printDescription();
					PswDialogView.enableOkButton();
					return;
				}
				// store session key:
				CipherStuff.getInstance().getSessionKeyCrypt().storeKey(keyMaterial);

				// create new Nonce and set
				byte[] nonce = new RandomStuff().createRandomBytes(Attachments.getNonceSize());
				Attachments.setNonce(nonce);

				// add files
				boolean added = decryptedFileTypePanel.getFileComposer().addAction(PswDialogView.getView(),
						PeaProperties.getBundle().getString("select_file_password_initialize"), 
						true, false);//, decryptedFileTypePanel.getLocationOnScreen());

				if (added == true) {
					//System.out.println("added");
					PswDialogView.clearPassword();			
					decryptedFileTypePanel.getFileComposer().getFileModel().calculateNumberAndSize(
							decryptedFileTypePanel.getValidFileNames(true, false));
					decryptedFileTypePanel.getFileComposer().updateFiles(false);	

					lockFrame.setVisible(true);		
					LockFrameFile.setFileDisplayPanel(decryptedFileTypePanel);					

					decryptedFileTypePanel.updateWindow();
					
				} else {
					System.out.println("no files added");
					PswDialogView.enableOkButton();
					return;
				}
				
				if (decryptedFileTypePanel.getFileComposer().getFileModel().checkForValidFile() == false){
					System.err.println("no valid files found");
					PswDialogView.enableOkButton();
					return;
				}
				PeaProperties.setTypePanel(decryptedFileTypePanel);	
				// close this view
				PswDialogView.getView().setVisible(false);	
				// encrypt by shutdown:
				decrypted = true;
				

				return;			
				
			} else { // not initializing

				// get files to encrypt:
				fileNames = PeaControl.getEncryptedFileTypePanel().getValidFileNames(true, false);

				if (fileNames == null || fileNames.length == 0) {
					System.err.println("No file was selected");
					PswDialogView.setMessage(PeaProperties.getBundle().getString("no_valid_file_selected"), true);
					PswDialogView.enableOkButton();
					return;
				} 

				boolean saltSuccess = setSaltForEncryptedFiles(fileNames);
				if (saltSuccess == false){
					PswDialogView.setMessage(
							"###" + // translate here, not in PswDialogView
									PeaProperties.getBundle().getString("file_inappropriate") + "\n" 
									//+ saltContainingFile//chosenFileNames[0] 
									//+ PeaProperties.getBundle().getString("caused_error")
									//+ e.toString()
									+ PeaProperties.getBundle().getString("attached_salt_not_found"), true);
					PswDialogView.enableOkButton();
					return;
				}


				// KDF:
				// derive the key from password and salt: 
				keyMaterial = getKeyMaterial();

				if (keyMaterial == null) {
					new UnexpectedValueException("keyMaterial", "byte[]", "is null").printDescription();
					PswDialogView.setMessage("Key derivation failed", true);
					PswDialogView.clearPassword();
					dialogView.pack();
					PswDialogView.enableOkButton();
					return;
				}
			}
		} catch (Exception ex) {

			System.err.println(ex.toString() + " - " + ex.getMessage() + "  program exits");
			ex.printStackTrace();
			System.exit(2);
		}


		FileModel encryptedFileModel = PeaControl.getEncryptedFileTypePanel().getFileComposer().getFileModel();
		// cleanup the map
		encryptedFileModel.cleanupMap();// remove invalid and deselected files
		if( encryptedFileModel.getAllSize() > (16 * 1024 * 1024)
				|| encryptedFileModel.getFileNumber() > 64) {
			decryptedFileTypePanel.startProgressTask();
		}
		decryptedFileTypePanel.getFileComposer().setPlainModus(false); // files are encrypted
		decryptedFileTypePanel.getFileComposer().addFilesToMap(new ArrayList<String>(Arrays.asList(fileNames)) , false, false, false);
		//for(int i=0;i<decryptedFileModel.getAllFiles().length;i++) System.out.println(decryptedFileModel.getAllFiles()[i].getAbsolutePath());


		String[] errorMessages = CipherStuff.getCipherMode().decryptFiles( fileNames, keyMaterial,
				true, decryptedFileTypePanel.getFileComposer() );
		decryptedFileTypePanel.getFileComposer().setPlainModus(true); // files are decrypted now

		FileModel decryptedFileModel = decryptedFileTypePanel.getFileComposer().getFileModel();
		
		// set annotations from errorMessages:
		int len = errorMessages.length;
		// to check if password failed
		boolean passwordFailed = false;
		for (int i = 0; i < len; i++) {
			String message = errorMessages[i];
			if (message != null) {
				if (printErrors == true){
					System.err.println(fileNames[i] + ": " + message);
				}
				if (message.startsWith("###")){ // do not translate
					decryptedFileModel.setAnnotation(
							new File(fileNames[i]), 
							message + FileModel.getInvalidMarker());

				} else {
					if (message.equals("password_failed")){
						passwordFailed = true;
					}
					decryptedFileModel.setAnnotation(
							new File(fileNames[i]), 
							PeaProperties.getBundle().getString(message) + FileModel.getInvalidMarker());
				}
			} else {
				// no error, valid file: overwrite messages from before
				// (wrong password)
				decryptedFileModel.setAnnotation(
						new File(fileNames[i]), "");
			}
		}

		// close progress task
		decryptedFileTypePanel.closeProgressTask();

		// check for valid file:
		if (decryptedFileModel.checkForValidFile() == false){
			if (passwordFailed == true) {
				PswDialogView.setMessage(PeaProperties.getBundle().getString("password_failed"), true);
			}			
			System.err.println("password failed for all files");
			decryptedFileModel.resetTreeMap();
			if(lockFrame.isValid()){
				lockFrame.setVisible(false);
			}
			PswDialogView.enableOkButton();
			return;
		}		


		System.out.println("file(s) decrypted");
		// encrypt by shutdown:
		decrypted = true;
		PeaProperties.setTypePanel(decryptedFileTypePanel);

		PswDialogView.clearPassword();			

		decryptedFileTypePanel.getFileComposer().updateFiles(false);

		//lockFrame.repaint();
		lockFrame.setVisible(true);		
		LockFrameFile.setFileDisplayPanel(decryptedFileTypePanel);
		
	//	if (decryptedFileTypePanel.getValidFileNames(true) != null) {
			decryptedFileModel.calculateNumberAndSize(decryptedFileTypePanel.getValidFileNames(true, false));	
	//	}

		decryptedFileTypePanel.updateWindow();


		PswDialogView.getView().setVisible(false);	
	}
	

	// used in PswDialogView to add selected file to path file:
	@Override
	public String[] getSelectedFileNames() {

		return ( (FileTypePanel)PeaProperties.getTypePanel()).getValidFileNames(true, false);

	}

	@Override
	public void initializeNewFiles() {
		// this is done by addAction in FileTypePanel when calling startDecryption with initializing == true
	}

	@Override
	public void openEncryptedFiles() {
		FileComposer encryptedComposer = PeaControl.getEncryptedFileTypePanel().getFileComposer();
		FileModel encryptedModel = encryptedComposer.getFileModel();
		boolean added = encryptedComposer.addAction(PeaControl.getEncryptedFileTypePanel(),
				PeaProperties.getBundle().getString("select_files_to_encrypt"), 
				true, true);// hidden files, check listed files
		if (added == false) { // no file was added
			System.err.println("no file was added");
			return;
		}
		
		boolean validFileFound = encryptedModel.checkForValidFile();
		if (validFileFound == false) {
			//PswDialogView.setMessage(PeaProperties.getBundle().getString("no_valid_file_found"), true);
			PswDialogView.setMessage(PeaProperties.getBundle().getString("open_or_initialize"), true);
		} else {
			// clear errorMessageLabel
			PswDialogView.setMessage("   ", true);
		}		
	}
	
	@Override
	public final LockFrameFile getLockFrame() {
		return lockFrame;
	}
	@Override
	public String getVersion() {
		return VERSION;
	}
	@Override
	public String getYearOfPublication() {
		return YEAR_OF_PUBLICATION;
	}
	@Override
	public String getWebsite() {
		return WEBSITE;
	}
	@Override
	public String getSourceLink() {
		return SOURCE_LINK;
	}

	@Override
	public boolean checkFormat(File file, boolean easyCheck, boolean middleCheck, boolean expensiveCheck) {
		// nothing to check: all formats are accepted
		return true;
	}
	@Override
	protected byte[] getPlainBytes() {
		// not used
		return null;
	}

	@Override
	public String setPeaPropValues(Properties prop, InputStream inputStream) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String updatePeaPropValues(Properties prop) {//, InputStream inputStream) {
		// TODO Auto-generated method stub
		return null;
	}
	
	//============ inner classes ================
	
	static class  EventDispatchThread implements Runnable{

		@Override
		public void run() {
			pswDialog = FileControl.getInstance();		

			dialogView.setVisible(true);
			//model = PeaControl.getEncryptedFileTypePanel().getFileComposer().getFileModel();
			if (PeaControl.getEncryptedFileTypePanel().getFileComposer().getFileModel().getFileNumber() == 0 ) {
				PswDialogView.setMessage(PeaProperties.getBundle().getString("open_or_initialize"), true);
			}			
		}		
	}
	
	/*
	 * Try to encrypt the opened files if  before the shutdown
	 */
	static class ShutdownHookThread extends Thread {

		public void run() {
			//System.err.println("shutdownhook");
			if (System.getProperty("os.name").contains("BSD")){
				// FreeBSD destroys the files in ShutdownHook...
				CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
			} else {
				if (decrypted == true) {
					if (LockFrameFile.getFileDisplayPanel().
							getFileComposer().getFileModel().getAllSize() 
							< (1024 * 4)) {// < 4 KB
						lockFrame.encryptCurrentFiles(null);
						CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
						decrypted = false;
					} else {
						// do not encrypt files: the execution time is long,
						// there is a risk of half-encrypting (damaging) the files
						CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
					}
				}
			}
		}
	}
}	