package cologne.eck.file_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main frame to display the selected files for file pea. 
 */




import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ArrayList;

import javax.accessibility.AccessibleContext;
import javax.swing.*;

import settings.PeaSettings;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.ExecutionTimeObserver;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.HelpMenu;
import cologne.eck.all_peas.gui.NewPasswordDialog;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PeaMenuBar;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.ViewMenu;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.tools.FileDropHandler;
import cologne.eck.tools.FileTools;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.Zeroizer;
import cologne.eck.tools.ZipStuff;



@SuppressWarnings("serial")
public class LockFrameFile extends JFrame 
		implements ActionListener,  WindowListener {

	private static LockFrameFile frame;// static instance
	private static FileTypePanel filePanel;// associated FileTypPanel to show the files
	private static JCheckBox rememberCheck = null; // remember the valid files
	
	private static JLabel messageLabel;

	
	private LockFrameFile(Point location, Dimension dim){
		
		frame = this;		
		//for(int i=0;i<selectedFileNames.length;i++) System.out.println("selected: " + selectedFileNames[i]);
		
	    this.setIconImage( PswDialogView.getImage() );		
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE); // do not allow to close or to exit:
															// files may remain unencrypted
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		this.setContentPane(contentPane);				
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		menuPanel.setMinimumSize(new Dimension(200, 30));
		menuPanel.setMaximumSize(new Dimension((int) PswDialogView.getScreenwidth(), 40));
		
		PeaMenuBar menuBar = new PeaMenuBar();
		//menuBar.setBorder(new EtchedBorder() );
		
		Font font = new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize());
		
		JMenu menu = new JMenu();
		menu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu for file operations.");
		menu.setFont(font);
		menu.setText(PeaProperties.getBundle().getString("menu"));//"Menu");
		menu.setMnemonic(KeyEvent.VK_M);
		menuBar.add(menu);
		
		JMenuItem encryptItem = new JMenuItem();
		encryptItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component encrypts and closes all selected files and exits.");
		encryptItem.setText(PeaProperties.getBundle().getString("close_and_encrypt_selected_files"));//"close and encrypt selected files");
		encryptItem.setActionCommand("encryptWithClosing");
		encryptItem.setFont(font);
		encryptItem.setMnemonic(KeyEvent.VK_E);
		encryptItem.addActionListener(this);
		menu.add(encryptItem);
		
		JMenuItem unencryptItem = new JMenuItem();
		unencryptItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component leaves selected files unencrypted and closes the program.");
		unencryptItem.setText(PeaProperties.getBundle().getString("close_and_leave_unencrypted"));//"close and leave all opened file unencrypted");
		unencryptItem.setActionCommand("leaveUnencrypted");
		unencryptItem.setFont(font);
		unencryptItem.setMnemonic(KeyEvent.VK_U);
		unencryptItem.addActionListener(this);
		menu.add(unencryptItem);
		
		JMenuItem addItem = new JMenuItem();
		addItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to add new plain text files.");
		addItem.setText(PeaProperties.getBundle().getString("add_new_files_to_encrypt"));//"add new files to encrypt");
		addItem.setActionCommand("addNewFiles");
		addItem.setFont(font);
		addItem.setMnemonic(KeyEvent.VK_E);
		addItem.addActionListener(this);
		menu.add(addItem);

		JMenuItem pswChangeItem = new JMenuItem();
		pswChangeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to change the password for opened files.");
		pswChangeItem.setText(PeaProperties.getBundle().getString("change_password"));//"change password");
		pswChangeItem.setActionCommand("changePsw");
		pswChangeItem.setFont(font);
		pswChangeItem.setMnemonic(KeyEvent.VK_P);
		pswChangeItem.addActionListener(this);
		menu.add(pswChangeItem);
		
		JMenu viewMenu = new ViewMenu(filePanel, true);// plainModus true
		menuBar.add(viewMenu);
		
		JMenu extraMenu = new JMenu();
		extraMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu for zip and wipe operations.");
		extraMenu.setFont(font);
		extraMenu.setText(PeaProperties.getBundle().getString("extra"));//"Menu");
		extraMenu.setMnemonic(KeyEvent.VK_E);
		menuBar.add(extraMenu);
		
		JMenuItem zipItem = new JMenuItem();
		zipItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component zippes the selected files.");
		zipItem.setText(PeaProperties.getBundle().getString("zip_and_add_selected_files"));
		zipItem.setActionCommand("zipSelection");
		zipItem.setFont(font);
		zipItem.setMnemonic(KeyEvent.VK_Z);
		zipItem.addActionListener(this);
		extraMenu.add(zipItem);
		
		JMenuItem unzipItem = new JMenuItem();
		unzipItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component unzips an archive and opens a menu to choose the folder to store.");
		unzipItem.setText(PeaProperties.getBundle().getString("choose_folder_to_zip"));;
		unzipItem.setActionCommand("zipFolder");
		unzipItem.setFont(font);
		unzipItem.setMnemonic(KeyEvent.VK_U);
		unzipItem.addActionListener(this);
		extraMenu.add(unzipItem);		
		
		JMenuItem wipeItem = new JMenuItem();
		wipeItem.setToolTipText("Overwrites file with 0, 1 and random data before deletion");
		wipeItem.setText(PeaProperties.getBundle().getString("wipe_and_delete"));
		wipeItem.setActionCommand("wipe");
		wipeItem.setFont(font);
		wipeItem.setMnemonic(KeyEvent.VK_W);
		wipeItem.addActionListener(this);
		extraMenu.add(wipeItem);
		
		JMenu helpMenu = new HelpMenu();
		menuBar.add(helpMenu);

		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		
		contentPane.add(Box.createVerticalStrut(5) );
		messageLabel = new JLabel("  ");
		//messageLabel.setForeground(Color.GREEN);
		contentPane.add(messageLabel);
		
		contentPane.add(Box.createVerticalStrut(20) );
		
		//
		// displayed Files:
		//
		filePanel.removeAddButton();// add button is not required because of the menu
		//filePanel.update(filePanel.getGraphics());
		// otherwise scroll bar disappears...
		filePanel.updateFileView();
		filePanel.repaint();
		filePanel.setTransferHandler(new FileDropHandler());
		contentPane.add(filePanel);		
		
		//filePanel.updateNumberAndSize();

		contentPane.add(Box.createVerticalStrut(10) );		
		
		JPanel checkRememberPanel = new JPanel();
		checkRememberPanel.setLayout(new BoxLayout(checkRememberPanel, BoxLayout.X_AXIS));
		
	    rememberCheck = new JCheckBox();	
	    rememberCheck.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
	    rememberCheck.setText(PeaProperties.getBundle().getString("remember_file_names"));//"remember selected file names");
	    rememberCheck.setToolTipText(PeaProperties.getBundle().getString("file_storage_info")// "file names will be saved in the file " +
	    		+  "  "  + PeaSettings.getJarFileName() + ".path");// in current directory");
		
	    checkRememberPanel.add(Box.createVerticalStrut(10));
	    checkRememberPanel.add(rememberCheck);
	    checkRememberPanel.add(Box.createHorizontalGlue() );

	    contentPane.add(checkRememberPanel);	
	    contentPane.add(Box.createVerticalStrut(10) );	
		
		JPanel noteLabelPanel = new JPanel();
		noteLabelPanel.setLayout(new BoxLayout(noteLabelPanel, BoxLayout.X_AXIS));
		JLabel noteLabel = new JLabel();
		noteLabel.setText(PeaProperties.getBundle().getString("selected_files_automatically_encrypted"));
		noteLabel.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 12));
		noteLabelPanel.add(noteLabel);
		noteLabelPanel.add(Box.createHorizontalGlue());
		contentPane.add(noteLabelPanel);
		
		this.setLocation(location);
		//this.setMinimumSize(new Dimension(400, 500));
		Dimension preferredDim = this.getPreferredSize();
		if (dim.height > preferredDim.height
				|| dim.width > preferredDim.width){
			this.setSize(dim);
		} else {
			pack();		
		}
	}

	/*
	 * Only one static instance allowed
	 */
	public final static LockFrameFile getInstance(Point loc, Dimension dim){

		// must create a new instance if password failed
		frame = new LockFrameFile(loc, dim);// selectedFileNames);	
		return frame;
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		String command = ape.getActionCommand();
		
		if ( command.equals("encryptWithClosing") ) {	

			closeProgram();
			
		} else if ( command.equals("leaveUnencrypted") ) {		
			
			messageLabel.setText("... " + PeaProperties.getBundle().getString("encryption") + " " +
					PeaProperties.getBundle().getString("in_progress") + "... ");
			messageLabel.repaint();
			messageLabel.getParent().revalidate();
			messageLabel.getParent().repaint();
			// display a warning and an option to break
			int result = JOptionPane.showConfirmDialog(this,
						PeaProperties.getBundle().getString("leave_files_unencrypted"),
					   PeaProperties.getBundle().getString("warning"),
					   JOptionPane.WARNING_MESSAGE,
					    JOptionPane.YES_NO_OPTION);
			if (result == 0) {	//yes = 0, 

				CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();	
				FileControl.setDecrypted(false);
				System.exit(0);
			} else { // no = 1, cancel = 2
				messageLabel.setText("");
				return;
			}			
			
		} else if ( command.equals("addNewFiles") ) {	
			
			try {
				messageLabel.setText("... " + PeaProperties.getBundle().getString("open_file") + " " +
						PeaProperties.getBundle().getString("in_progress") + "... ");

			} finally {
				//FileComposer composer = filePanel.getFileComposer();
				filePanel.getFileComposer().addAction(this,
						PeaProperties.getBundle().getString("select_files_to_encrypt"), 
						true, true);	// hidden file option = true, check again, = true
				// adding with checkAgain=true results in wrong fileNumber:
				filePanel.getFileComposer().getFileModel().calculateNumberAndSize(filePanel.getValidFileNames(true, false));
				filePanel.updateNumberAndSize();

				messageLabel.setText("");
			}

		} else if ( command.equals("changePsw") ) {		
//System.out.println("thread: " +javax.swing.SwingUtilities.isEventDispatchThread());
			String[] unselectedFiles = filePanel.getUnselectedValidFileNames();
			if (unselectedFiles == null){
				// nothing to do
			} else {
				if (unselectedFiles.length == 0){
					// nothing to do
				} else {
					// there are unselected valid files:
					// do not allow to change the password 
					JOptionPane.showMessageDialog(frame, 
							PeaProperties.getBundle().getString("remaining_files_for_password_change"),
							PeaProperties.getBundle().getString("error"), 
							JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
		
			NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
					this, PeaProperties.getBundle().getString("change_password"), null,
					frame.getLocation());
			char[] newPsw = newPswDialog.getDialogInput();

			//==============================================
			// new Nonce for every new password
			byte[] nonce = new RandomStuff().createRandomBytes(Attachments.getNonceSize());
			Attachments.setNonce(nonce);
			//==============================================

			try {
				messageLabel.setText("... " + PeaProperties.getBundle().getString("change_password") + " " +
						PeaProperties.getBundle().getString("in_progress")  + "... ");
			} finally {

				byte[] keyMaterial = FileControl.deriveKeyFromPsw(newPsw);
				if (newPsw != null){
					Zeroizer.zero(newPsw);
				}						
				CipherStuff.getInstance().encryptKeyFromKeyMaterial(keyMaterial);
				messageLabel.setText("");//PeaProperties.getBundle().getString("change_password") + " - " +
				//PeaProperties.getBundle().getString("done"));
			}
		} else if ( command.equals("zipSelection") ) {	
			//System.out.println("zip selection");			
			
			// get selected file names, if no, break with message
			String[] fileNames = filePanel.getValidFileNames(true, false);
			if (fileNames == null || fileNames.length == 0) {
				PeaDialog.showMessage(this, PeaProperties.getBundle().getString("no_valid_file_selected"), "Zip", 0);
				return;
			}
			if (FileTools.getNumberAndSize(fileNames)[1] > 1024*1024*128) { // > 256MB
				if (ExecutionTimeObserver.showWarningAndBreakOption(
			    		this, PeaProperties.getBundle().getString("size_warning"), false) 
						== false) {
					return;
				}
			}
			
			// get name of zip file to create, if no, break with message
			String zipName = null;
			zipName = PeaDialog.showInputDialog(this, PeaProperties.getBundle().getString("name_of_zip"), "Zip", -1);
			
			if (zipName == null || zipName.equals("")) {
				PeaDialog.showMessage(this, "Zip name is required", "Zip", 0);
				return;
			} else {			
				zipName = zipName + "_pea.zip";
				String newZipName = zipName.replaceAll("[^a-zA-Z0-9\\.\\-]", "_");
				if (! newZipName.equals(zipName)) {
					PeaDialog.showMessage(this, "Problematic characters for file name were replaced: \n"
							+ newZipName, null, 0);
					zipName = newZipName;
				}				
			}
			
			// select directory to store zip file
			File zipFolder;
			PeaFileChooser chooser = new PeaFileChooser(this);
			chooser.customizeFileChooser("Zip", PeaProperties.getBundle().getString("select_folder_to_store_zip"), false);
			chooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY);
			//In response to a button click:
			int returnVal = chooser.showOpenDialog(this);
			if (chooser.getSelectedFile() == null || returnVal != JFileChooser.APPROVE_OPTION) {
				PeaDialog.showMessage(this, "You must choose a folder", "Zip", 0);
				return;					
			} else {
				zipFolder = chooser.getSelectedFile();
			}

			// zip
			String errorMessage = null;
			try {//  Wait cursor is also set in ZipStuff
				messageLabel.setText("... " + PeaProperties.getBundle().getString("zip") + " " +
						PeaProperties.getBundle().getString("in_progress") + "...");
				// set the wait cursor: 
				frame.setCursor(CursorManager.getWaitCursor());
			} finally {

				errorMessage = ZipStuff.zipFiles(fileNames, false, false, zipName, 
						zipFolder, (Component) PeaProperties.getTypePanel(), PeaProperties.getBundle());	

				frame.setCursor(CursorManager.getDefaultCursor());
				messageLabel.setText("");
			}

			if (errorMessage != null) {
				PeaDialog.showMessage(this, errorMessage, "Zip", 0);
				return;
			} else {
				System.out.println("Zip file created: " + zipFolder + File.separator + zipName);
			}
			int len = fileNames.length;
			
			FileModel model = filePanel.getFileComposer().getFileModel();
			// delete original files?
			String fileNamesString = "";
			for (int i = 0; i < len; i++) {
				fileNamesString += fileNames[i] + "\n";
			}
			int deleteFiles = PeaDialog.showQuestionWithScollPane(
					getFileDisplayPanel(), 
					PeaProperties.getBundle().getString("zip_created_delete_original") + "\n" + fileNamesString,
					"Zip", 0, 3);

			if (deleteFiles == 0) { // delete ok
				String errorMessageWipe = "";
				try {
					messageLabel.setText("... " + PeaProperties.getBundle().getString("wipe_and_delete") + " " +
							PeaProperties.getBundle().getString("in_progress") + "... ");
					// set the wait cursor: 
					frame.setCursor(CursorManager.getWaitCursor());

				} finally {
					// list directories: if they become empty, delete
					ArrayList<String> dirToDelete = new ArrayList<String>();
					for (int i = 0; i < len; i++) {
						long size = FileTools.getFileSize(fileNames[i]);
						String newError = FileTools.wipeFile(fileNames[i], true);
						if (newError == null) {
							filePanel.setCheckBoxInvalid(fileNames[i], "deleted");
							model.setAnnotation(new File(fileNames[i]), "deleted" + FileModel.getInvalidMarker());
							// subtract original size (now size is 0)
							model.increaseAllSize( -size);
						} else {
							if (newError.equals("is folder")) {
								dirToDelete.add(fileNames[i]);
							} else {
								errorMessageWipe += "\n" + newError;
							}
						}
					}
					// manually delete remaining directories if they are empty
					int foldersLen = dirToDelete.size();
					for (int i = foldersLen-1; i >=0; i--) {// begin with last: file hierarchy is from parent folders
						File folder = new File(dirToDelete.get(i));
						// delete only if there are no remaining files
						if (folder.exists() && (FileTools.getNumberAndSize(dirToDelete.get(i))[1] == 0)){
							folder.delete();
							filePanel.setCheckBoxInvalid(dirToDelete.get(i), "deleted");
							model.setAnnotation(folder, "deleted" + FileModel.getInvalidMarker());
						}
					}
					frame.setCursor(CursorManager.getDefaultCursor());
					messageLabel.setText("");
				}
				
				filePanel.updateNumberAndSize();
				if (errorMessageWipe.length() > 0) {
					PeaDialog.showMessage(PeaProperties.getTypePanel(), "Deletion failed: \n" + errorMessageWipe, "Wipe", 0);
				} else {
					PeaDialog.showMessage(PeaProperties.getTypePanel(), PeaProperties.getBundle().getString("files_wiped"), "Wipe", 1);					
				}
			}
			// add zip file to FileTypePanel
			File zipFile = new File(zipFolder.getAbsolutePath() + File.separator + zipName);

			// ad or update file:
			filePanel.getFileComposer().addOrUpdateFile(zipFile, false);		
			filePanel.updateNumberAndSize();
			filePanel.updateFileView();			
			
		} else if ( command.equals("zipFolder") ) {	
			System.out.println("zip folder");			
			// select a file to wipe:
			PeaFileChooser chooser = new PeaFileChooser(this);
			chooser.customizeFileChooser("Zip", PeaProperties.getBundle().getString("select_folder_to_zip"), false);
			chooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY);
			//In response to a button click:
			int returnVal = chooser.showOpenDialog(this);
			if (chooser.getSelectedFile() == null || returnVal != JFileChooser.APPROVE_OPTION) {
				System.out.println("No folder selected...");
				return;					
			} else {
				File folder = chooser.getSelectedFile();

				String errorMessage = null;
				try { //  Wait cursor is also set in ZipStuff
					// set the wait cursor: 
					frame.setCursor(CursorManager.getWaitCursor());
					messageLabel.setText("... " + PeaProperties.getBundle().getString("zip") + " " +
							PeaProperties.getBundle().getString("in_progress") + "... ");
				} finally {					
					errorMessage = ZipStuff.zipFolder(folder.getAbsolutePath(), folder.getName(), this, PeaProperties.getBundle());			
					frame.setCursor(CursorManager.getDefaultCursor());
					messageLabel.setText("");
				}
				
				
				if (errorMessage != null) {
					PeaDialog.showMessage(this, PeaProperties.getBundle().getString("zip_failed") + ": \n" + errorMessage, "Zip", 0);
				} else {
					PeaDialog.showMessage(this, PeaProperties.getBundle().getString("zip_created") + ": \n" + folder.getAbsolutePath() + ".zip"
							+ "\n" + PeaProperties.getBundle().getString("beside_folder"), "Zip", 1); 
					// add or update file:
					filePanel.getFileComposer().addOrUpdateFile(new File(folder.getAbsolutePath() + ".zip"), false);		
					filePanel.updateNumberAndSize();
					filePanel.updateFileView();		
				}
			}
			
		} else if ( command.equals("wipe") ) {	
			
			// select a file to wipe:
			PeaFileChooser chooser = new PeaFileChooser(this);
			chooser.customizeFileChooser("Wipe", PeaProperties.getBundle().getString("select_file_to_wipe"), false);
			chooser.setFileSelectionMode( JFileChooser.FILES_ONLY);
			//In response to a button click:
			int returnVal = chooser.showOpenDialog(this);
			if (chooser.getSelectedFile() == null || returnVal != JFileChooser.APPROVE_OPTION) {
				System.out.println("No file wiped and deleted...");
				return;					
			} else {
				File file = chooser.getSelectedFile();
				long size = FileTools.getFileSize(file);
				String errorMessage = null;
				try {
					messageLabel.setText("... " + PeaProperties.getBundle().getString("wipe_and_delete") + " " +
							PeaProperties.getBundle().getString("in_progress") + "... ");
				} finally {
					errorMessage = FileTools.wipeFile(file.getAbsolutePath(), true);
					messageLabel.setText("");
				}
				if (errorMessage == null) {
					PeaDialog.showMessage(this, PeaProperties.getBundle().getString("file_wiped") + ": \n" + file.getAbsolutePath(), "Wipe", 1);
				} else {
					PeaDialog.showMessage(this, "Deletion failed: \n" + file.getAbsolutePath()
					+"\n" + errorMessage, "Wipe", 0);
					return;
				}
				// check if map contains file, then mark as invalid and subtract allSize manually
				FileModel model = filePanel.getFileComposer().getFileModel();
				if (model.containsFile(file)){
					// remove file from map: 
					filePanel.setCheckBoxInvalid(file.getAbsolutePath(), "deleted");
					model.setAnnotation(file, "deleted" + FileModel.getInvalidMarker());
					// subtract original size (now size is 0)
					model.increaseAllSize( -size);
					filePanel.updateNumberAndSize();
					filePanel.updateFileView();
				}
			}			
		}
	}
	

	
	
	/**
	 * Handle unselected files, encrypt files, clear
	 * keys, set decrypted for shutdown hook and exit
	 */
	private void closeProgram(){

		// if unselected files remain: warning
		if (cancelClosingProcess() == true) {
			// closing process was canceled because of
			// unencrypted remaining files
			return;
		}
		try {
			messageLabel.setText(PeaProperties.getBundle().getString("change_password") + " " +
					PeaProperties.getBundle().getString("in_progress"));

			// set the wait cursor: 
			frame.setCursor(CursorManager.getWaitCursor());

		} finally {
			encryptCurrentFiles(null);
			// clear the sessionKeys:
			CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();	

			FileControl.setDecrypted(false);

			new PropertyHandler().updateAllPropValues();
			System.exit(0);
		}
	}
	
	@Override
	public void windowClosing(WindowEvent we) { // by X-Button

		closeProgram();
	}
	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}
	
	/*
	 * Encrypt the selected files
	 */
	public final void encryptCurrentFiles(byte[] keyMaterial){

		String[] fileNames = filePanel.getValidFileNames(true, false);
		
		if (fileNames == null || fileNames.length == 0){
			System.err.println("No files to encrypt selected");
			FileControl.setDecrypted(false);
			return;
		}

		// remember: add files to path file
		if (rememberCheck.isSelected() == true) {
			PathFileManager.addFilesToPathFile(fileNames);
		}
		// give informations if there has been errors:
		String errors = null;
		if(filePanel.getFileComposer().getFileModel().getAllSize() > (16 * 1024 * 1024)
				|| filePanel.getFileComposer().getFileModel().getFileNumber() > 64) {

			filePanel.startProgressTask();
		}
		try {
			// set the wait cursor: 
			frame.setCursor(CursorManager.getWaitCursor());
			
			String[] errorMessages = CipherStuff.getCipherMode().encryptFiles(
					fileNames, keyMaterial, true, filePanel.getFileComposer());
			filePanel.closeProgressTask();// close automatically

			for (int i = 0; i < errorMessages.length; i++) {
				if (errorMessages[i] != null) {
					errors +=  errorMessages[i] + " - " + fileNames[i] + "\n";
				}
			}	
		} finally {
			frame.setCursor(CursorManager.getDefaultCursor());
		}
		if (errors != null) {
			JOptionPane.showMessageDialog(filePanel, 
					PeaProperties.getBundle().getString("error_in_decryption_process")
					+ errors, 
					PeaProperties.getBundle().getString("error"),                                
					JOptionPane.ERROR_MESSAGE);    
		}
		FileControl.setDecrypted(false);
		System.out.println("file(s) encrypted");
	}

	/**
	 * Show a warning message if there are file 
	 * left which will remain unencrypted
	 * 
	 * @return	true, if the closing process should be 
	 * 			canceled because of unencrypted files,
	 * 			false if closing process can go on
	 */
	private final boolean cancelClosingProcess(){
		// if unselected files remain: warning
		String[] unselectedFiles = filePanel.getUnselectedValidFileNames();
		if ( unselectedFiles != null ) {
			// create String to display:
			StringBuilder unselectedFilesBuilder = new StringBuilder();
			for (int i = 0; i < unselectedFiles.length; i++) {
				unselectedFilesBuilder.append(unselectedFiles[i]);
				unselectedFilesBuilder.append("\n");
			}
			String unselectedFilesString = new String(unselectedFilesBuilder);
//System.out.println("Warning remaining: "+ unselectedFilesString);

			int result = PeaDialog.showQuestionWithScollPane(this, 
					PeaProperties.getBundle().getString("unselected_files_left")
					+ "\n\n" + unselectedFilesString, 
					PeaProperties.getBundle().getString("warning"), 
					2, // ok-cancel option
					2);// warning message
			if (result == 0) { //JOptionPane.OK_OPTION
				// unencrypted files were accepted, go on
				return false;
			} else { // cancel, close
				// don't close...
//System.out.println("break closing");
				return true;
			}
		} else {
			// nothing to do, go on
			return false;
		}
	//	FileControl.setDecrypted(false);
	//	return true; // close
	}

	
	//==================================
	// Getter & Setter
	public final static FileTypePanel getFileDisplayPanel() {
		return filePanel;
	}
	public final static void setFileDisplayPanel(FileTypePanel _fdp) {
		filePanel = _fdp;
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameFile();
        }
        return accessibleContext;
    }

    protected class AccessibleLockFrameFile extends AccessibleJFrame {
        //Inherit everything, override nothing.
    }
}